package de.w665.sharepulse.service;

import de.w665.sharepulse.db.repo.FileUploadRepository;
import de.w665.sharepulse.model.FileUpload;
import jakarta.annotation.PostConstruct;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.tomcat.util.http.fileupload.FileUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Date;

@Slf4j
@Service
public class FileService {

    private final FileIdService fileIdService;

    private final FileUploadRepository fileUploadRepository;

    @Value("${sharepulse.temp-filestore-path}")
    private String tempDirPath;

    @Autowired
    public FileService(FileIdService fileIdService, FileUploadRepository fileUploadRepository) {
        this.fileIdService = fileIdService;
        this.fileUploadRepository = fileUploadRepository;
    }

    @PostConstruct
    public void createFolder() {
        Path path = Paths.get(getTempDirPath());
        log.debug("Checking temp file directory " + path.getFileName());
        try {
            if (!Files.exists(path)) {
                Files.createDirectory(path);
                log.debug("Directory created");
            } else {
                log.debug("Directory already exists. Clearing content.");
                FileUtils.cleanDirectory(new File(getTempDirPath()));
                log.debug("Directory content cleared.");
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public String processUploadedFile(MultipartFile file, String uploaderIp, String password, boolean singleDownload, String fileDescription) {

        String fileId = fileIdService.generateNewUniqueId();

        FileUpload fileUpload = FileUpload.builder()
                .fileId(fileId)
                .fileName(file.getOriginalFilename())
                .fileSize(file.getSize())
                .singleDownload(singleDownload)
                .uploadDate(new Date())
                .uploadedByIpAddress(uploaderIp)
                .downloadCount(0)
                .fileDescription(fileDescription)
                .passwordProtected(password != null && !password.isEmpty())
                .downloadPassword(password)
                .build();

        System.out.println(fileUpload.toString());

        fileUploadRepository.insertFileUpload(fileUpload);


        // TODO: rename file to fileID
        Path path = Paths.get(getTempDirPath() + File.separator + fileId);
        try {
            Files.write(path, file.getBytes());
            log.debug("File " + file.getOriginalFilename() + " written to " + path.getFileName());
            return fileId;
        } catch (IOException e) {
            e.printStackTrace();
            return null;
        }
    }

    public File getFileById(String fileId) {
        return new File(getTempDirPath() + File.separator + fileId);
    }

    public FileUpload getFileUploadByFileId(String fileId) {
        return fileUploadRepository.retrieveFileUploadByFileId(fileId);
    }

    public long updateDownloadCount(FileUpload fileUpload) {
        fileUpload.setDownloadCount(fileUpload.getDownloadCount() + 1);
        fileUploadRepository.updateFileUpload(fileUpload);
        return fileUpload.getDownloadCount();
    }


    private String getTempDirPath() {
        return System.getProperty("user.dir") + tempDirPath;
    }


}
